import 'package:diameter/components/detail.dart';
import 'package:diameter/components/dialogs.dart';
import 'package:diameter/components/dropdown.dart';
import 'package:diameter/components/forms.dart';
import 'package:diameter/models/basal_profile.dart';
import 'package:diameter/models/bolus_profile.dart';
import 'package:diameter/models/log_event_type.dart';
import 'package:diameter/models/settings.dart';
import 'package:diameter/navigation.dart';
import 'package:flutter/material.dart';

class EventTypeDetailScreen extends StatefulWidget {
  static const String routeName = '/log-event-type';
  final int id;
  const EventTypeDetailScreen({Key? key, this.id = 0}) : super(key: key);

  @override
  _EventTypeDetailScreenState createState() =>
      _EventTypeDetailScreenState();
}

class _EventTypeDetailScreenState extends State<EventTypeDetailScreen> {
  LogEventType? _logEventType;
  bool _isNew = true;
  bool _isSaving = false;

  List<BolusProfile> _bolusProfiles = [];
  List<BasalProfile> _basalProfiles = [];

  final GlobalKey<FormState> _logEventTypeForm = GlobalKey<FormState>();

  final _valueController = TextEditingController(text: '');
  final _defaultReminderDurationController = TextEditingController(text: '');
  final _notesController = TextEditingController(text: '');
  bool _hasEndTime = false;
  BolusProfile? _bolusProfile;
  BasalProfile? _basalProfile;

  @override
  void initState() {
    super.initState();

    reload();

    _bolusProfiles = BolusProfile.getAll();
    _basalProfiles = BasalProfile.getAll();

    if (_logEventType != null) {
      _valueController.text = _logEventType!.value;
      _defaultReminderDurationController.text =
          (_logEventType!.defaultReminderDuration ?? '').toString();
      _hasEndTime = _logEventType!.hasEndTime;
      _notesController.text = _logEventType!.notes ?? '';
      _basalProfile = _logEventType!.basalProfile.target;
      _bolusProfile = _logEventType!.bolusProfile.target;
    }
  }

  void reload() {
    if (widget.id != 0) {
      setState(() {
        _logEventType = LogEventType.get(widget.id);
      });
    }
    _isNew = _logEventType == null;
  }

  void handleSaveAction() async {
    setState(() {
      _isSaving = true;
    });
    if (_logEventTypeForm.currentState!.validate()) {
      LogEventType eventType = LogEventType(
        id: widget.id,
        value: _valueController.text,
        notes: _notesController.text,
        defaultReminderDuration:
            int.tryParse(_defaultReminderDurationController.text),
        hasEndTime: _hasEndTime,
      );
      eventType.basalProfile.target = _basalProfile;
      eventType.bolusProfile.target = _bolusProfile;
      LogEventType.put(eventType);
      Navigator.pop(context, '${_isNew ? 'New' : ''} Log Event Type Saved');
    }
    setState(() {
      _isSaving = false;
    });
  }

  void handleCancelAction() {
    bool isNew = _logEventType == null;
    if (Settings.get().showConfirmationDialogOnCancel &&
        ((isNew &&
                (_valueController.text != '' ||
                    int.tryParse(_defaultReminderDurationController.text) !=
                        null ||
                    _notesController.text != '' ||
                    _hasEndTime)) ||
            (!isNew &&
                (_valueController.text != _logEventType!.value ||
                    int.tryParse(_defaultReminderDurationController.text) !=
                        _logEventType!.defaultReminderDuration ||
                    _notesController.text != (_logEventType!.notes ?? '') ||
                    _hasEndTime != _logEventType!.hasEndTime)))) {
      Dialogs.showCancelConfirmationDialog(
        context: context,
        isNew: isNew,
        onSave: handleSaveAction,
      );
    } else {
      Navigator.pop(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(_isNew ? 'New Log Event Type' : _logEventType!.value),
      ),
      drawer:
          const Navigation(currentLocation: EventTypeDetailScreen.routeName),
      body: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            FormWrapper(
              formState: _logEventTypeForm,
              fields: [
                TextFormField(
                  controller: _valueController,
                  decoration: const InputDecoration(
                    labelText: 'Name',
                    alignLabelWithHint: true,
                  ),
                  validator: (value) {
                    if (value!.trim().isEmpty) {
                      return 'Empty name';
                    }
                    return null;
                  },
                ),
                BooleanFormField(
                  value: _hasEndTime,
                  label: 'has end time',
                  onChanged: (value) {
                    setState(() {
                      _hasEndTime = value;
                    });
                  },
                ),
                Column(
                  children: _hasEndTime ? [
                    TextFormField(
                      controller: _defaultReminderDurationController,
                      keyboardType: const TextInputType.numberWithOptions(),
                      decoration: InputDecoration(
                        labelText: 'Default Reminder Duration',
                        suffixText: ' min',
                        enabled: _hasEndTime,
                      ),
                    ),
                  AutoCompleteDropdownButton<BolusProfile>(
                  selectedItem: _bolusProfile,
                  label: 'Bolus Profile',
                  items: _bolusProfiles,
                  onChanged: (value) {
                    setState(() {
                      _bolusProfile = value;
                    });
                  },
                ),
                    AutoCompleteDropdownButton<BasalProfile>(
                  selectedItem: _basalProfile,
                  label: 'Basal Profile',
                  items: _basalProfiles,
                  onChanged: (value) {
                    setState(() {
                      _basalProfile = value;
                    });
                  },
                ),
              ] : []),
                TextFormField(
                  controller: _notesController,
                  decoration: const InputDecoration(
                    labelText: 'Notes',
                    alignLabelWithHint: true,
                  ),
                  keyboardType: TextInputType.multiline,
                ),
              ]
            ),
          ],
        ),
      ),
      bottomNavigationBar: DetailBottomRow(
        onCancel: handleCancelAction,
        onSave: _isSaving ? null : handleSaveAction,
      ),
    );
  }
}
